<?php
/**
 * @package   com_zoo
 * @author    YOOtheme http://www.yootheme.com
 * @copyright Copyright (C) YOOtheme GmbH
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * The related items element class
 * Class ElementRelatedProducts
 */
class ElementRelatedProducts extends Element implements iSubmittable
{

    protected $_related_items;

    /**
     * Checks if the element's value is set.
     * @param array $params
     * @return bool
     */
    public function hasValue($params = array())
    {
        return false;
    }

    /**
     * Renders the element
     * @param array $params
     * @return string
     */
    public function render($params = array())
    {
        return null;
    }

    /**
     * @param bool $published
     * @return array
     */
    public function getRelatedItems($published = true)
    {
        if ($this->_related_items == null) {

            // init vars
            $table                = $this->app->table->item;
            $this->_related_items = array();
            $related_items        = array();

            // get items
            $items = $this->get('item', array());

            // check if items have already been retrieved
            foreach ($items as $key => $id) {
                if ($table->has($id)) {
                    $related_items[$id] = $table->get($id);
                    unset($items[$key]);
                }
            }

            if (!empty($items)) {
                // get dates
                $db           = $this->app->database;
                $date         = $this->app->date->create();
                $now          = $db->Quote($date->toSQL());
                $null         = $db->Quote($db->getNullDate());
                $items_string = implode(', ', $items);
                $conditions   = $table->key . ' IN (' . $items_string . ')'
                    . ($published ? ' AND state = 1'
                        . ' AND ' . $this->app->user->getDBAccessString()
                        . ' AND (publish_up = ' . $null . ' OR publish_up <= ' . $now . ')'
                        . ' AND (publish_down = ' . $null . ' OR publish_down >= ' . $now . ')' : '');
                $order        = 'FIELD(' . $table->key . ',' . $items_string . ')';
                $related_items += $table->all(compact('conditions', 'order'));
            }

            foreach ($this->get('item', array()) as $id) {
                if (isset($related_items[$id])) {
                    $this->_related_items[$id] = $related_items[$id];
                }
            }

        }

        return $this->_related_items;
    }

    /**
     * Renders the edit form field.
     * @return string
     */
    public function edit()
    {
        return $this->_edit(false);
    }

    /**
     * Renders the element in submission.
     * @param array $params AppData submission parameters
     * @return string
     */
    public function renderSubmission($params = array())
    {
        // load assets
        $this->app->document->addScript('elements:relateditems/relateditems.js');

        return $this->_edit();

    }

    /**
     * @param bool $published
     * @return string
     */
    protected function _edit($published = true)
    {
        $query = array('controller' => 'item', 'task' => 'element', 'tmpl' => 'component', 'func' => 'selectRelateditem', 'object' => $this->identifier);

        // filter types
        foreach ($this->config->get('selectable_types', array()) as $key => $selectable_type) {
            $query["type_filter[$key]"] = $selectable_type;
        }

        // filter items
        if ($this->_item) {
            $query['item_filter'] = $this->_item->id;
        }

        if ($layout = $this->getLayout('edit.php')) {
            return $this->renderLayout($layout,
                array(
                    'data' => $this->getRelatedItems($published),
                    'link' => $this->app->link($query)
                )
            );
        }
    }

    /**
     * Validates the submitted element
     * @param $value AppData value
     * @param $params AppData submission parameters
     * @return array
     * @throws AppValidatorException
     */
    public function validateSubmission($value, $params)
    {
        $options  = array('required' => $params->get('required'));
        $messages = array('required' => 'Please select at least one related item.');

        $items = (array)$this->app->validator
            ->create('foreach', $this->app->validator->create('integer'), $options, $messages)
            ->clean($value->get('item'));

        $table = $this->app->table->item;
        if ($selectable_types = $this->config->get('selectable_types', array()) and !empty($selectable_types)) {
            foreach ($items as $item) {
                if (!empty($item) && !in_array($table->get($item)->type, $selectable_types)) {
                    throw new AppValidatorException('Please choose a correct related item.');
                }
            }
        }

        return array('item' => $items);
    }

    /**
     * Load elements css/js assets
     * @return $this|void
     */
    public function loadAssets()
    {
        $this->app->document->addScript('elements:relatedproducts/relatedproducts.js');
    }

    /**
     * Get parameter form object to render input form.
     * @return null
     */
    public function getConfigForm()
    {
        return parent::getConfigForm()->addElementPath(dirname(__FILE__));
    }

}